@@ -7,5 +7,5 @@ extern crate lazy_static;
 pub mod parser;
 pub mod utils;
 
-pub use parser::parse_env;
+pub use parser::{list_available_keywords, parse_env, resolve_keyword};
 pub use utils::{read_env_file, write_env_file, Config, EnvContents};
